;*********************************************************************
;
;Purpose: Demo code showing how to delete records from the database
;         and then traverse through the database ignoring deleted 
;         records.
;
;Paul Squires (2000-2003)
;
;*********************************************************************

 
;Define the names of the database & index
    DBFname.s = "Cheetah.dbf"
    IDXname.s = "Cheetah.idx"

;Create the database
    AllFields.s = "CUSTID,C,7,0;CUSTNAME,C,35,0;SALARY,N,14,2"
    xdbCreate(DBFName, AllFields)
 
;Open the database (database must be open prior To creating index)
    dbHandle.l = xdbOpen(DBFName, "") ; no encryption = ""
    Debug xdbAnyErrors() 

;Create the index (database must be open)
   IndexExpr.s = "UPPER(CUSTID)"   ;index is not case sensitive
   Duplicates.l = #XDBTRUE         ;allow duplicate customer ID's
    
   xdbCreateIndex(IDXName, dbHandle, IndexExpr, Duplicates)

;Open the index
   idxHandle.l = xdbOpenIndex(IDXname, dbHandle)
 
   msg.s = "DBFname: " + DBFname + Chr(10) + "dbHandle:" +  Str(dbHandle) + Chr(10) + Chr(10) + "IDXname: " + IDXname + Chr(10) + "idxHandle:" + Str(idxHandle)
   MessageRequester("", msg, 0)


;Add records to the database and index
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;NOTE: When adding a large number of records it is better to close
;the index, do the appends to the database and then open the
;index and reindex. For a few hundred records there should be no
;performance issues.

NumRecs.l = 200

Msg.s = "Press 'Okay' To create " + Str(NumRecs) + " database and index these records."
MessageRequester("PureBasic",msg, 0)

; Predefine the field#'s for speed. If you already know the
; which field name corresponds to which field number then
; there is no need to do this. These functions provide
; really fast lookups.
   CustIDfield.l = xdbFieldNumber(dbHandle, "CUSTID")
   CustNameField.l = xdbFieldNumber(dbHandle,"CUSTNAME")
   SalaryField.l = xdbFieldNumber(dbHandle, "SALARY")
  
  For x.l = 1 To NumRecs

      xdbClearBuffer(dbHandle)  ;this will clear the record buffer
     ;NOTE: You must do this step to assure clean information
          
      CustID.s = RSet(Trim(Str(x)), 7)
           
      xdbAssignField(dbHandle, "", CustIDfield, CustID)
       
      xdbAssignField(dbHandle, "", CustNameField, "The big Cat")
   
      xdbAssignField(dbHandle, "", SalaryField, "-1234.50")
      
     ;Add to the end of the database (Append) & add the key to the index.
      xdbAddRecord(dbHandle)  
      
  Next x

;Get RecCount
   Result = xdbRecordCount(dbHandle)
   msg.s = "Database Records BEFORE delete: " + Str(Result)
   MessageRequester("PureBasic", msg)
 
; Delete 3 records. The record must first be loaded into the record buffer
; before it can be marked for deletion. Records are not physically removed
; from the database until xdbPack or xdbZap is called. This gives you the
; opportunity to recall (unmark from deletion) records using the xdbRecall
; function call.
      xdbDeleteRecord(dbHandle, 100)    ;mark record 100 for deletion
      xdbDeleteRecord(dbHandle, 101)    ;mark record 101 for deletion
      xdbDeleteRecord(dbHandle, 102)    ;mark record 102 for deletion
 
; notice the number of records have not changed. The records are only marked
; for deletion. If you are moving through the database and want to exclude
; processing any deleted records then you should test the record using the
; xdbDeleted& function or set the xdbSkipDeleted function to %XDBTRUE.

;Get RecCount
   Result = xdbRecordCount(dbHandle)
   msg.s = "Database Records AFTER delete: " + Str(Result) + Chr(10) + "NOTE: Includes records marked for deletion."
   MessageRequester("PureBasic", msg)

; now move through the database ignoring all deleted records (using the
; idxHandle& index)
   Msg.s = "Move through records ignoring deleted records (xdbDeleted)."
   MessageRequester("PureBasic", msg)
  
    xdbMoveFirst(dbHandle, idxHandle)

    NumProcessed.l = 0
  
Repeat

   RecNo.l = xdbRecordNumber(dbHandle)
       If xdbDeleted(dbHandle, RecNo)
;           this record is deleted so skip it!
       Else
;           process the record
         NumProcessed = NumProcessed + 1
       EndIf
     xdbMoveNext(dbHandle, idxHandle)
     
Until xdbEOF(dbHandle)

      msg.s = "Processed " + Str(NumProcessed) + "  non-deleted records in the database."
      MessageRequester("PureBasic", msg) 

; Now move through the database ignoring all deleted records (using the
; idxHandle& index). Use the alternate method of using xdbSkipDeleted.
      msg.s = "Move through records ignoring deleted records (xdbSkipDeleted)."
      MessageRequester("PureBasic", msg)

    xdbSkipDeleted(dbHandle, #XDBTRUE)
    xdbMoveFirst(dbHandle, idxHandle)

NumProcessed = 0

Repeat
    NumProcessed = NumProcessed + 1
    xdbMoveNext(dbHandle, idxHandle)
    
Until xdbEOF(dbHandle)
    
     msg.s = "Processed " + Str(NumProcessed) + " non-deleted records in the database."
     MessageRequester("PureBasic", msg)

; Pack the database. This will physically remove all records marked for
; deletion. The changes are permanent - records can not be recalled after
; this function is called. Any open indexes are automatically reindexed.
    xdbPack(dbHandle)
 
; The record count should now be properly reflected.
     Result = xdbRecordCount(dbHandle)
      msg.s = "Database Records AFTER Pack: " + Str(Result)
      MessageRequester("PureBasic", msg)

; ZAP the database. This instantly deletes all records and related index
; keys. You can not recover from this process so it is advisable to make
; backups of the database and indexes prior to calling this function. You
; do not have to manually call the idxReindex routine.
; 
    xdbZap(dbHandle)
; 
; the record count should now be zero.
      Result = xdbRecordCount(dbHandle)
      msg.s = "Database Records AFTER ZAP: " + Str(Result)
      MessageRequester("PureBasic", msg)

;Close the database and related index
 xdbClose(dbHandle)                                 

End
; ExecutableFormat=Windows
; CursorPosition=127
; FirstLine=121
; EOF