;*********************************************************************
; 
; Purpose: Demo code showing how to retrieve the values of each field
;          in a record and how to update those field values and store
;          the results in the database (updating any changed key fields
;          as necessary).
; 
; Paul Squires (2000-2003)
; 
;*********************************************************************


;Define the names of the database & index
   DBFname.s = "Cheetah.dbf"
   IDXname.s = "Cheetah.idx"

Title.s = "PureBasic Cheetah Database"    

;Create the database
   AllFields.s = "CUSTID,C,7,0;CUSTNAME,C,35,0;SALARY,N,14,2"
   xdbCreate(DBFName, AllFields)
 
;Open the database (database must be open prior To creating index)
    dbHandle.l = xdbOpen(DBFName, "") ; no encryption = ""
    Debug xdbAnyErrors() 

;Create the index (database must be open)
   IndexExpr.s = "UPPER(CUSTID)"   ;index is not case sensitive
   Duplicates.l = #XDBTRUE         ;allow duplicate customer ID's
    
   xdbCreateIndex(IDXName, dbHandle, IndexExpr, Duplicates)

;Open the index
   idxHandle.l = xdbOpenIndex(IDXname, dbHandle)

;Add records to the database and index
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;NOTE: When adding a large number of records it is better to close
;the index, do the appends to the database and then open the
;index and reindex. For a few hundred records there should be no
;performance issues.

   NumRecs.l = 200
   Msg.s = "Press 'Okay' To create " + Str(NumRecs) + " database and index these records."
   MessageRequester("PureBasic",msg, 0)

; ------------------------------------------------------------------------
; Predefine the field#'s for speed. If you already know the
; which field name corresponds to which field number then
; there is no need to do this. These functions provide
; really fast lookups.
;------------------------------------------------------------------------
   CustIDfield.l = xdbFieldNumber(dbHandle, "CUSTID")
   CustNameField.l = xdbFieldNumber(dbHandle,"CUSTNAME")
   SalaryField.l = xdbFieldNumber(dbHandle, "SALARY")
  
   For x.l = 1 To NumRecs

      xdbClearBuffer(dbHandle)  ;this will clear the record buffer
     ;NOTE: You must do this step to assure clean information
          
      CustID.s = RSet(Trim(Str(x)), 7)
           
      xdbAssignField(dbHandle, "", CustIDfield, CustID)
      xdbAssignField(dbHandle, "", CustNameField, "Billy Gates" + Str(x))
      xdbAssignField(dbHandle, "", SalaryField, "100234.50")
      
     ;Add to the end of the database (Append) & add the key to the index.
      xdbAddRecord(dbHandle)  
      
   Next x

   msg.s = "Database and Index records created." + Chr(10) + "Next, retrieve record field values..."
   MessageRequester(Title, msg, 0)


; The database and index now contains records. Get one of the records, check its
; field values, change a value and post it back to the database (which automatically
; updates any open indexes if the key field was changed).
; 
; Get a record. This automatically loads the RecordBuffer with the record's Data.
; I'll retrieve And update the first record in the database.
; 
   RecordNumber.l = 100
  
; Instead of using xdbGetRecord you could also have used any of the xdbMove
; functions. When the position in the database/index is changed using the
; xdbMove functions, the RecordBuffer is automatically loaded correctly.
; 
; You can view the raw recordbuffer using the xdbRecordBuffer function. However,
; it is recommended that you use the xdbFieldValue functions (refer to the
; Cheetah Help file for syntax usage).
; 
; retrieve the field values from the current record. I will use the field numbers
; because they are already known. You could just as easily pass the field name
; instead. There is barely any difference in speed because Cheetah uses an
; EXTREMELY fast lookup for fieldnames! 

   xdbGetRecord(dbHandle, RecordNumber)  ;the recordbuffer is now loaded
 
; using the FieldNumber&
  CustID.s = xdbFieldValue(dbHandle, "", CustIDfield)
; OR use the FieldName instead of the FieldNumber&.
;    as in:  CustID.s = xdbFieldValue(dbHandle, "CUSTID", 0)
 
  CustName.s = xdbFieldValue(dbHandle, "", CustNamefield)
; OR CustName$ = xdbFieldValue$(dbHandle&, "CUSTNAME", 0)

  Salary.s = xdbFieldValue(dbHandle, "", SalaryField)
; OR Salary$ = xdbFieldValue$(dbHandle&, "SALARY", 0)
 
; Display the values
 msg.s = "Record #" + Str(xdbRecordNumber(dbHandle)) + " Field Values:" 
 msg.s = msg + Chr(10)
 msg.s = msg + "CustID: " + CustID + Chr(10)
 msg.s = msg + "CustName: " + CustName + Chr(10)
 msg.s = msg +  "Salary: "  + Salary  +Chr(10)
 msg.s = msg +  "Now, change the CustName and store it back in the database...."
   MessageRequester (Title, msg, 0)  

;Update the record in the record buffer And store it back in the database. This
;will automatically update the index if necessary (in this case the key, CustID, is
;not being changed so the index will not be updated).

;store new CustName$. This updates the RecordBuffer (not the actual database)
   xdbAssignField(dbHandle, "CUSTNAME", 0, "Paul Squires")

;store the RecordBuffer in the database/index.
   xdbPutRecord(dbHandle, RecordNumber)

msg.s = "The record has been updated. Now read the database again" 
msg = msg + " to ensure that the changes are there..." + Chr(10)
msg = msg + "This is rec #:" + Str(RecordNumber)
   MessageRequester (Title, msg, 0)  



;verify that the values were written correctly.
   xdbGetRecord(dbHandle, RecordNumber)  ;the recordbuffer is now loaded

CustID.s = xdbFieldValue(dbHandle, "CUSTID", 0)
CustName.s = xdbFieldValue(dbHandle, "CUSTNAME", 0)
Salary.s = xdbFieldValue(dbHandle, "", SalaryField)

;display the values
        msg.s = "Record #" + Str(xdbRecordNumber(dbHandle)) + " Field Values:"+ Chr(10) + Chr(10)
        msg = msg + "CustID: " + CustID + Chr(10)
        msg = msg +  "CustName: " + CustName + Chr(10)
        msg = msg +  "Salary: " + Salary + Chr(10)
        msg = msg +  "That's it! ...."

   MessageRequester (Title, msg, 0)  

;Close the database And related index
   xdbClose(dbHandle)                                 
  

End


; ExecutableFormat=Windows
; CursorPosition=44
; FirstLine=22
; EOF